#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2013 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;

namespace HeuristicLab.Grammars {
  public class AttributedGrammar : Grammar {
    private readonly Dictionary<ISymbol, string> localDefinitions;
    private readonly Dictionary<ISymbol, List<Sequence>> alternatives;

    public AttributedGrammar(ISymbol startSymbol, IEnumerable<ISymbol> nonTerminals, IEnumerable<ISymbol> terminals)
      : base(startSymbol, nonTerminals, terminals) {
      this.StartSymbol = startSymbol;
      localDefinitions = nonTerminals.ToDictionary(nt => nt, nt => string.Empty);
      alternatives = nonTerminals.ToDictionary(nt => nt, nt => new List<Sequence>());
    }

    public IEnumerable<Sequence> GetAlternativesWithSemanticActions(ISymbol ntSymbol) {
      return alternatives[ntSymbol];
    }
    public Sequence GetAlternativeWithSemanticActions(ISymbol ntSymbol, int index) {
      return alternatives[ntSymbol][index];
    }

    public override void AddProductionRule(ISymbol ntSymbol, Sequence production) {
      base.AddProductionRule(ntSymbol, new Sequence(production.Where(symb => !(symb is SemanticSymbol)).ToList()));

      IList<Sequence> list = alternatives[ntSymbol];

      // make sure that there is not equivalent production registered already
      Debug.Assert(!list.Any(s => s.SequenceEqual(production)));

      list.Add(production);
    }

    public void AddLocalDefinitions(ISymbol ntSymbol, string localDefCode) {
      Debug.Assert(!ntSymbol.Equals(EmptySymbol));

      localDefinitions[ntSymbol] = localDefCode;
    }
    public string GetLocalDefinitions(ISymbol ntSymbol) {
      return localDefinitions[ntSymbol];
    }
  }
}